#!/apps/CA/siteminder/CLI/bin/perl
  eval 'exec perl -S $0 ${1+"$@"}' if 0;
#----------------------------------------------------------------------------
#   Copyright © 2018 CA. All rights reserved.  
#
#   This software and all information contained therein is confidential and
#   proprietary and shall not be duplicated, used, disclosed or disseminated
#   in any way without the express written permission of CA. All authorized
#   reproductions must be marked with this language.  
#
#   TO THE EXTENT PERMITTED BY APPLICABLE LAW OR AS AGREED BY CA IN ITS 
#   APPLICABLE LICENSE AGREEMENT, CA PROVIDES THIS DOCUMENTATION “AS IS” 
#   WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY IMPLIED 
#   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR 
#   NONINFRINGEMENT. IN NO EVENT WILL CA BE LIABLE TO THE END USER OR ANY 
#   THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF
#   THIS DOCUMENTATION, INCLUDING WITHOUT LIMITATION, LOST PROFITS, LOST 
#   INVESTMENT, BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA 
#   IS EXPRESSLY ADVISED IN ADVANCE OF THE POSSIBILITY OF SUCH LOSS OR
#   DAMAGE.
#
#----------------------------------------------------------------------------
#
#   sm_idle_timeouts.pl   Richard J. Faust   2018-05-15
#
#   This script will modify the idle timeout for all protected realms in the 
#   specified domain to the value provided. 
#   
#   The script still produces some informational messages at the terminal and
#   also write results to a text log file file.
#
#   $ ./sm_idle_timeouts_v0.1.7_2018-09-26 [ARGV[0] ... ARGV[4] [--mode={audit|modify} [--verbose]
#
#   Optional arguments are:   
#       ARGV[0] - New idle timeout, in seconds
#       ARGV[1] - domains file.  This is the list of domains for which 
#                 the realm timeouts should be modified, or a single domain
#                 name.  This argument also has a special, case-sensitive
#                 keyword.  Process all legacy domains if:
#                     ARGV[1]=GetAllDomains!
#                 This should be quite useful during auditing runs to 
#                 retrieve all timeouts, import the output file into Excel,
#                 filter as desired, then create a Domains.txt file for more
#                 focused modification of idle timeouts.
#       ARGV[2] - organization name.
#       ARGV[3] - environment, such as {DEV|QA|PROD}.  Valid only if the
#                 organization name is also specified.
#       ARGV[4] - a single domain name.  Intended for debugging purposes to 
#                 verify script output for just one domain.
#
#   The output file is tab-delimited and suitable for being imported into an 
#   Excel spreadsheet.  The name of the output file follows this format:
#
#   sm_idle_timeouts{_ARGV[2]}{_ARGV[3]}_YYYY-MM-DD_HH-MM-SS.txt
#
#   where:
#       ARGV[2] and ARGV[3] are define above. The organization name must be
#               provided as ARGV[2] for the environment provided as ARGV[3] to be
#               included in the output file name.
#    YYYY-MM-DD is the ISO standard date format at the instant data reporting
#               begins.
#    HH-MM-SS   is the hour, minute and second (local time) at the instant data
#               reporting begins.
#
#   Command options:
#
#   -m, --mode = { audit | create }
#       audit:   This is the default mode.  Update object names and values 
#			according to command line arguments and/or answers provided to the 
#			prompts, but don't create any new objects in the r12 configuration.
#
#       modify:  update the idle timeout of realms which exceed the value
#           specified on the command line.  Each and every realm to be modified
#           must first be confirmed by the user.
#
#	-v, --verbose
#		Display additional information and diagnostic messages.
#
#   Revision history:
#   2018-08-31 Rich Faust - initial version... cloned a copy of the Group Responses script 
#   2018-09-07 Rich Faust - refine code for processing multiple domains
#   2018-09-10 Rich Faust v0.1.3
#       - loop for additional domain or file with a list of one or more domains
#   2018-09-10 Rich Faust v0.1.4
#       - Update comments
#       - write output to logfile
#   2018-09-11 Rich Faust v0.1.5
#       - add command options:
#           -m or --mode={audit|modify}, default=audit
#           -v or --verbose, if present, display additional information messages
#       - Modify idle timeout only if the idle timeout specified on the command
#         line is less than the existing idle timeout for the protected realms
#         in the requested domains.
#   2018-09-25 Rich Faust v0.l.6
#       - update comments
#       - remove hard-coded password
#   2018-09-26 Rich Faust v0.1.7
#       - Add case-sensitive keyword for ARGV[1]:  GetAllDomains!
#         Using this keyword will result in ALL legacy domains being processed.
#
#-----------------------------------------------------------------------------

use strict;
use warnings;

my $scriptVersion = "v0.1.7 2018-09-26";

print "\n$0 starting at ";
system ("date");
print "\n";

use Getopt::Long;
use Netegrity::PolicyMgtAPI;

my $nrec = -1;
my $npass = 0;
my $debug = 0;
my $compressed_record;
my $adminName;
my $adminPwd;
my $password;
my $sttime;
my $policymgtapi;
my $session;
my $entime;
my $elapse;
my @realms;
my $domain;
my @domain_names;
my $realm;
my @domains;
my $domain_name;
my $domainName;
my $domains_to_process;
my $domain_scalar;
my $total_domains = 0;
my $total_realms = 0;
my $total_realms_modified = 0;
my $open_status;
my $status;
my $nactive;
my $realmName;
my $resourceFilter;
my $maxTimeout;
my $idleTimeout;
my $nrealm;
my $report_file_name;
my $organizationName;
my $environmentName;
my $mode       = 'audit';  # this is the default mode
my @validModes = qw( audit modify );
my $verbose    = 0;

#-----------------------------------------------------------------------------

sub getPassword{
    my $flush = $|;
    $| = 1;  # flush output to screen byte by byte
 
    my $opt = shift || "";
    $opt = substr $opt,0,1;

    my ($bs, $ws, $bullet) = ("\b", " ", $opt);
    ($bs, $ws, $bullet) = ("", "", "") unless $opt;

    my $pass = "";
    while (1) {
        my $ch = getone();
   
        # Return character, end of getting password
        $ch =~ /[\r\n]/ && do {
            print "\n";
            $| = $flush;  # return to previous setting
            return $pass;
        };
   
        # Backspace/delete key entered
        $ch =~ /[\b\x7F]/ && do {
            if (length ($pass) > 0) {
                chop $pass;
                print "$bs$ws$bs";
            }
            next;
        };
   
        # Warn for non-ascii characters
        if (ord($ch) < 32) {
            print "\7";
            next;
        };

        # Valid password character enetered
        $pass .= $ch;
        print $bullet;
    }
}
#---------------------------------------------------------------------
#    STDIN_QUESTION
#
#    If no arguments assume yes and no answers.  returns a y or n
#
#    $options     Holds the list of valid options.
#    $text        Hold the text to be displayed.
#    $full_match  Tests for full match
#
#---------------------------------------------------------------------
sub stdin_question {
    #Get arguments
    my ($options,$text,$match) = @_;    #Get passed in var

    #Variables
    my $input;
    my $flag = 0;
    my @valid_options;

    #Check for definition of options
    if (!defined($options)) {
        $options = "yes,no";
    }

    #Check for definition of text
    if (!defined($text)) {
        $text = "(yes or no) :";
    }

    #Check for defination of match
    if (!defined($match)) {
        $match = 0;
    }

    #Put options in an array
    @valid_options = split/,/, $options;

    #While no match loop
    while ($flag ne 1) {
        #Print out std text
        print $text;

        #Read and chomp stdin
        $input = <stdin>;
        chomp $input;

        #Loop through array for match
        foreach (@valid_options) {
            #Test for match
            if (/$input/i) {
                #Test for full match to compare on length
                if ($match eq 1) {
                    if (length($_) eq length($input)) {
                        $flag = 1;
                    }
                } else {
                    $flag = 1;
                }
            }
        }
    }

    #Return answer
    return lc $input;
}

#-----------------------------------------------------------------------------
#   Get admin username and password to create a policy server session.

BEGIN {
    use POSIX qw(:termios_h);

    my ($term, $oterm, $echo, $noecho, $fd_stdin);

    $fd_stdin = fileno(STDIN);

    $term     = POSIX::Termios->new();
    $term->getattr($fd_stdin);
    $oterm     = $term->getlflag();

    $echo     = ECHO | ECHOK | ICANON;
    $noecho   = $oterm & ~$echo;

    sub cbreak {
        $term->setlflag($noecho);
        $term->setcc(VTIME, 1);
        $term->setattr($fd_stdin, TCSANOW);
    }

    sub cooked {
        $term->setlflag($oterm);
        $term->setcc(VTIME, 0);
        $term->setattr($fd_stdin, TCSANOW);
    }

    sub getone {
        my $key = '';
        cbreak();
        sysread(STDIN, $key, 1);
        cooked();
        return $key;
    }
}

END { cooked() }

print "admin username: ";
chomp($adminName = <STDIN>);

print "admin password: ";
chomp($adminPwd = $password = getPassword("*"));

print "\n";

# End site-specific configuration
#-----------------------------------------------------------------------------

sub get_domains {

    $nactive=-1;

    if ($domains_to_process eq "GetAllDomains!") {
        @domains=$session->GetAllDomains();
        $nactive = scalar @domains;
        print "\nNumber of domains to process: " . ($nactive+1) . "\n\n";
        return;
    }

    $open_status = open (ADF, "<", "$domains_to_process") or $open_status=0;

#    print "\n\$open_status = $open_status\n";

    if ($open_status eq 0) {

        $domain_names[0]=$domains_to_process;
        $nactive += 1;

    } else {

        print "\nDomains file opened:  $domains_to_process\n";

        while (<ADF>) {

#   Skip empty records and comment records.

            $compressed_record = $_;
            $compressed_record =~ s/\s+//g;

            if ((substr($_,0,1) eq "!") || (length($compressed_record) == 0)) {
                if ($debug) {
                    print "record commented out or blank: $_\n";
                }
                next;
            }

        	chomp ($_);

            $nactive += 1;
            $_ =~ s/\s+$//;     # trim trailing whitespace from the domain name
            $domain_names[$nactive] = $_;

        	next;
        }

        close ADF;

    } 

    print "\nNumber of domains to process: " . ($nactive+1) . "\n\n";

    my $ndomain = -1;

    foreach $domain_name(@domain_names) {

        $domain=$session->GetDomain($domain_name);

        if (defined $domain && $domain ne '') {

            $ndomain+=1;
            $domains[$ndomain]=$domain;
            if ($verbose) {
                print "\n\$domains[$ndomain] = " . $domains[$ndomain] . "($domain_name)";
            }
        } else {
            print "\n* * * Skipping undefined file or domain: $domain_name * * *\n";
        }
    }
}

#-----------------------------------------------------------------------------
# Get the list of domains.  Abort if the list is not provided or the list is
# empty.


print "\nGet the list of domain names to process...\n";

if ((defined $ARGV[1]) and ($ARGV[1] ne '')) {
    $domains_to_process = $ARGV[1];
    $domains_to_process =~ s/^\s+|\s+$//g;
    print "\nPer the command line, the domain file/name  =  $domains_to_process\n";
} else {
    $domains_to_process = '';
    print "\nEnter a domain name or the name of file containing a list of one or more domain names\n[Domains.txt]: ";
    chomp($domains_to_process = <STDIN>);
    $domains_to_process =~ s/^\s+|\s+$//g;
    if (length $domains_to_process == 0) {
        $domains_to_process="Domains.txt";
    }
}

if (defined $ARGV[0]) {
    print "\nNew idle timeout (seconds): " . $ARGV[0] . "\n";
}

#----------------------------------------------------------------------------
#	Determine whether we should just audit idle timeouts (mode=audit) or
#   prompt to modify idle timeouts (mode=modify). 

GetOptions (
    'mode=s'             => \$mode,
    'verbose!'           => \$verbose
);

$mode       =~ tr/[A-Z]/[a-z]/;     # ensure the mode is lower case

if ($verbose) {print "\$verbose = $verbose\n";}

#-----------------------------------------------------------------------------
# Create the session.

print "\nCreate session...\n";

$sttime = time;

$policymgtapi = Netegrity::PolicyMgtAPI->New();
$session = $policymgtapi->CreateSession($adminName, $adminPwd, "127.0.0.1");

if (! $session) {
    die "\nFATAL: Cannot create session. Please check admin credentials\n";
}

$entime = time;
$elapse = $entime - $sttime;
print "\nSession created in ".$elapse." seconds\n";

my $new_idle_timeout=$ARGV[0];

#-----------------------------------------------------------------------------
# Compose logfile and open a new log file for write.

my $logfile="sm_idle_timeouts";

if ($ARGV[2] ne "") {
    $logfile = $logfile . "_$ARGV[2]";
}

if ($ARGV[3] ne "") {
    $logfile = $logfile . "_$ARGV[3]";
}

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
my $now = sprintf("_%04d-%02d-%02d_%02d-%02d-%02d", $year+1900, $mon+1, $mday, $hour, $min, $sec);
#my $now = sprintf("_%04d-%02d-%02d", $year+1900, $mon+1, $mday);

$logfile = $logfile . $now . ".txt";

print "\n\$logfile = $logfile\n\n";

open (LOGFILE , "+>", $logfile) || die $!;

my $log_record;

#-----------------------------------------------------------------------------
# Process Domain Objects

while ($npass >= 0) {

$npass += 1;

$status = get_domains ();

if(@domains) {
    print "\n\nSUCCESS: requested domain(s) retrieved";
} else {
    print "\nWARNING: Unable to retrieve \@domains array \n";
    print "\n" . '=' x 78;
    print "\nEnter a domain name or the name of file containing a list of one or more domain\nnames or <RETURN> to exit [<RETURN>]: ";
    chomp($domains_to_process = <STDIN>);

    $domains_to_process =~ s/^\s+|\s+$//g;
    next;
}

print "\n\nModify idle timeouts for all realms...";

foreach $domain(@domains) {
    $total_domains += 1;
    if (defined $domain && $domain ne '') {
        $domainName = $domain->Name();
        print "\n" . '-' x 78 . "\nProcess domain: $domainName";
    } else {
        print "\nSkipping undefined domain";
        next;
    }

    $nrealm = -1;
    undef @realms;
    @realms = $domain->GetAllRealms();
    foreach $realm(@realms) {
        $nrealm += 1;
        $total_realms += 1;
        if (defined $realm && $realm ne '') {
            $realmName      = $realm->Name();
            $resourceFilter = $realm->ResourceFilter();
            print "\n  Process realm: " . $realmName . " = " . $resourceFilter;
        } else {
            print "\n  Skipping undefined realm in $domainName";
            next;
        }

        if ( $realm->ProtectResource() ) {         # Process only protected realms
            $maxTimeout  = $realm->MaxTimeout();
            $idleTimeout = $realm->IdleTimeout();
            print " (" . $maxTimeout . "," . $idleTimeout . ")\n";
            $log_record = "$domainName\t$realmName\t$resourceFilter\t$maxTimeout\t$idleTimeout";

            if ($mode eq "modify") {
                if ($idleTimeout <= $new_idle_timeout) {
                    print "\n  WARNING: skipping this realm since existing idle timeout:  $idleTimeout";
                    print "\n           is less than or equal to new idle timeout:        $new_idle_timeout\n";
                } else {
                    if (stdin_question('y,n','  Modify idle timeout {(Y)es|(N)o}? ') =~ /y/) {
                        $idleTimeout = $realm->IdleTimeout($new_idle_timeout);
                        print "\n  New timeouts: (" . $maxTimeout . "," . $idleTimeout . ")\n";
                        $total_realms_modified += 1;
                        $log_record = "$log_record\t$maxTimeout\t$idleTimeout";
                    }
                }
            }
        
            my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
            my $now = sprintf("%04d-%02d-%02d\t%02d:%02d:%02d\t", $year+1900, $mon+1, $mday, $hour, $min, $sec);
            print LOGFILE $now . $log_record . "\n";
            if ($verbose) {
                print "  \$log_record: " . $now . $log_record . "\n";
            }
        }
    }
}

print "\n\n";

#   Prompt for a new domain name or domain list.  Initialize arrays prior to 
#   next pass or display statistics and exit.

print "\n" . '=' x 78;
print "\nEnter a domain name or the name of file containing a list of one or more domain\nnames or <RETURN> to exit [<RETURN>]: ";
chomp($domains_to_process = <STDIN>);

$domains_to_process =~ s/^\s+|\s+$//g;

if (length $domains_to_process > 0) {
    undef @domain_names;
    undef @domains;
    next;
} else {

    close LOGFILE;

    print "\nNumber of passes:        $npass";
    print "\nTotal domains processed: $total_domains";
    print "\nTotal realms processed:  $total_realms";
    print "\nTotal realms modified:   $total_realms_modified\n";
    print "\n$0 finished at ";
    system ("date");
    print "\n\n";

    exit 0;
}

}   # close scope for loop to process domains

